from _TutorialBase import *

class MySitePage(HTMLPage):

    query_vars = [('view', '0', int)]
    use_session = True

    css = '''
    #banner {
       padding: 10px;
       background:aqua;
       margin-bottom: 10px;
    }
    #banner_content {
       padding: 10px;
       border: 1px solid;
    }
    '''

    # site page has a banner across the top followed by content
    def write_body_parts(self):
        self.writeln('<DIV id="banner">')
        self.write_banner()
        self.writeln('</DIV>')
        self.write_content()

    def write_banner(self):

        # banner content
        self.writeln(expand_codes(banner))

        # show source if requested
        if self.view in (1,2):
            self.writeln('<a href="%s">Hide Source</a>' % self.req.uri)
            if self.view == 1:
                src = file(self.sourcefilename()).read()
            else:
                src = file(os.path.join(os.path.dirname(self.req.filename),
                                        'SiteStuff.py')).read()
            self.writeln(HTMLHighlightedPy(src))


class MySiteAuthPage(MySitePage):

    # form variables to manage authentication
    form_vars = ['logout', 'username', 'passwd']
    
    def prep(self):
        MySitePage.prep(self)

        # requesting logout?
        if self.logout:
            self.authenticated = self.session['auth'] = None
        else:
            # get authentication from the session
            self.authenticated = self.session.get('auth')

        # if not authenticated, see if username/passwd pair are correct
        if (self.authenticated is None and self.username == 'mpservlets' and
            self.passwd == 'authdemo'):

            # OK, authenticate user and store in session
            self.authenticated = self.session['auth'] = True
        
        
    def write_html(self):

        # If necessary, save the current write_content and replace
        # with genloginform.
        if not self.authenticated:
            save, self.write_content = self.write_content, self.genloginform

        # In case of an exception place the call to write_html in
        # a try/finally suite so we can restore write_html.  We do
        # this because servlet instances live for multiple
        # requests and we want to make sure the correct
        # write_content method is installed.
        try:
            MySitePage.write_html(self)
        finally:
            # If necessary, restore write_content.
            if not self.authenticated:
                self.write_content = save

        return True

    def genloginform(self):

        # create form
        form = FORM(method="POST")
        form.append('Username: ',
                    INPUT(name="username", value=self.username),
                    ' (mpservlets)', BR(), BR(),
                    
                    'Password: ',
                    INPUT(type="password", name="passwd"),
                    ' (authdemo)', BR(), BR(),

                    INPUT(type='submit', value='Login'))

        # write it out
        self.writeln(form)


# banner content
banner = DIV(

    DIV(DIV(H3('Authentication By Subclassing'), align='center'),

        P('''Lets say you have an abstract base servlet, MySitePage,
        which manages the general layout of your site, a template, as
        it were.  You can then create another abstract servlet,
        MySiteAuthPage, which inherits directly from MySitePage and
        requires authentication before showing its content.  It does
        this by first checking the session to see if its authenticated
        and, if not, momentarily replaces C{write_content} with a
        different method that shows a login form.'''),

        P('''Thats what we have here.  The file
        A{SiteStuff.py<?view=2>} defines MySitePage and
        MySiteAuthPage.  Every servlet subclasses MySitePage if no
        authentication is required, or MySiteAuthPage if a login is
        necessary.  There are two servlets in this demo:'''),

        DefinitionList(('A{sitenoauth<sitenoauth>}',
                        '''This servlet subclasses MySitePage and
                        requires no authentication'''),

                       ('A{siteauth<siteauth>}', '''This servlet
                        subclasses MySiteAuthPage and does require
                        authentication''')),

        P('''A{View<?view=1>} the source of this servlet.'''),

        id='banner_content')

    )

