import os.path
from mod_python.servlet import HTMLPage
from _HTMLHighlightedPy import HTMLHighlightedPy
from _HTML import *

import thread

class TutorialBase(HTMLPage):

    py = {}

    meta = {'author' : 'Daniel J. Popowich',
            'copyright' : '2004'}

    tutorials = ['index', 'nooverride', 'hypertext',
                 'ivars', 'more_ivars', 'formdata',
                 'qvars', 'qvars2', 'conversion',
                 'lists', 'dicts', 'qvarsfvars',
                 'sessions', 'auth', 'auth2',
                 'lifecycle', 'okmethods', 'overriding',
                 'overriding2', 'fini']

    def __repr__(self):
        return self.__str__()

    def prep(self):

        HTMLPage.prep(self)
        self.src = self.form.getfirst('view')
        this = os.path.basename(self.req.uri)
        try:
            index = self.tutorials.index(this)
        except:
            self.nav = (None, None)
            self.index = -1
        else:
            self.nav = ((index and [index - 1] or [None])[0],
                        index < len(self.tutorials) - 1 and index+1 or None)
            self.index = index + 1

    def write_nav(self):
        prev, next = self.nav
        view = self.src and '?view=1' or ''

        self.writeln((prev is not None
                      and A('Previous', href=self.tutorials[prev]+view)
                      or 'Previous'), ' ',
                     (next is not None
                      and A('Next', href=self.tutorials[next]+view)
                      or 'Next'), ' ')


    def write_body(self):
        '''Write out the BODY.

           Normally, this method would not be overridden, but since
           this tutorial wants to demonstrate overriding
           write_body_parts, we need to do special things here and
           leave write_body_parts free to be overridden in tutorial
           examples.

           '''

        # the following is copied verbatim from the base class
        body_attrs = []
        for attr, val in self.body_attrs.items():
            body_attrs.append('%s="%s"' % (attr, val))
        self.writeln('<body %s>' % ' '.join(body_attrs))

        # now add the TITLE, index links and Hide/View Source links
        title = self.title
        if callable(title):
            title = title()
        self.writeln(CENTER(H1(title)))

        index = CENTER()
        for x in range(len(self.tutorials)):
            if x+1 == self.index:
                index.append('%d&nbsp;&nbsp;' % (x+1))
                continue
            index.append(A(x+1, href=self.tutorials[x]), '&nbsp;&nbsp;')
        self.writeln(index, BR())

        if self.src:
            self.writeln(A('Hide Source ', href=self.req.uri),
                         '(%s)' % self.sourcefilename())
            self.writeln(self.py.setdefault(self,
                        HTMLHighlightedPy(open(self.sourcefilename()).read())))
        else:
            self.writeln(A('View Source ', href=('%s?view=1' % self.req.uri)),
                         '(%s)' % self.sourcefilename())

        # write out navigation and HR
        self.writeln(BR())
        self.write_nav()
        self.writeln(HR())

        # now call write_body_parts
        self.write_body_parts()

        # closing HR and navigation
        self.writeln(HR())
        self.write_nav()
        
        # now close the body
        self.writeln('</body>')

def expand_codes(s):

    '''Expand codes in string to HTML elements.

    For any text of the form X{text}, replace it with
    <ELEMENT>text</ELEMENT>, where X and related elements are::

      C  <CODE>...</CODE>
      E  <EM>...</EM>
      S  <STRONG>...</STRONG>

    or A{TEXT<URL>} will generate: <A HREF="URL">TEXT</A>

    Inspired by epytext.  This simple version, however, does not
    allow embedding codes within each other.  :-(
    '''

    import re

    # so we can pass in non-string objects
    s = str(s)

    # iterate over our codes
    for code, elem in (('C', 'CODE'),
                       ('E', 'EM'),
                       ('S', 'STRONG')):

        s = re.sub(r'%s{(.*?)}' % code,
                   r'<%s>\1</%s>' % (elem, elem),
                   s)

    s = re.sub(r'A{(.*?)<(.*?)>}',
               r'<a href="\2">\1</a>',
               s)

    return s
