# -*- python -*-

from _TutorialBase import *

def _age(age):

    'Converts age (a string) to an int.  All values < 1 return as -1'

    try:
        a = int(age)
    except:
        # bad input, return our flag
        return -1
    
    if a < 1:
        # return our flag
        return -1
    return a

class conversion(TutorialBase):

    title = 'Converting Query Variables to Arbitrary Types'
    css_links = ['tutorial.css']

    query_vars = ['firstname',
                  ('age', '-1', _age)] # convert using _age function

    def write_content(self):

        if self.req.method == 'POST':
            self.writeln('Show me the ',
                         A('overview', href=self.req.uri),
                         ' again.')
            self.process_form()
            self.writeln(BR())
        else:
            self.writeln(expand_codes(overview))

        self.writeln(self.theform())
        

    def process_form(self):
        div = DIV(klass='formoutput')
        div.append('This servlet has been posted via a form:', BR(), BR())
        if self.firstname and self.age != -1:
            div.append('The form was filled out properly!', BR(), BR(),
                       'First Name: ', self.firstname, BR(),
                       'Age: ', self.age, BR())
        else:
            if not self.firstname:
                div.append('First name was not specified.', BR())
            if self.age == -1:
                div.append('Age was not specified properly.', BR())

        self.writeln(CENTER(div))

    def theform(self):
        table = TABLE(TR(TD('First name: '),
                         TD(INPUT(type='text', name='firstname',
                                  value=self.firstname, size=20))),
                      TR(TD('Age: '),
                         TD(INPUT(type='text', name='age',
                                  value=(self.age == -1 and ['']
                                         or [self.age])[0],
                                  size=3))),
                      TR(TD(INPUT(type='submit', name='Submit Form'),
                            colspan=2, align='center')),
                      klass='formdisplay')
                  
        return FORM(CENTER(table), method='POST')


overview = DIV(

    P('''By default, all data returned via an URL or the posting of a
      form are strings.  This can make for tedious programming having
      to manually convert the incoming strings to other python types.
      Properly setting C{query_vars} can eliminate much of this tedium
      by allowing the developer to specify exactly what types each
      instance variable should be by specifying a default value and/or
      providing a conversion function.'''),

    P('''This servlet will demonstrate how to convert strings to
      arbitrary types by specifying a conversion function.  The next
      servlets will demonstrate special treatment for python lists and
      dicts.'''),

    P('''If you look at the API reference you will see that
      C{query_vars} can be set to a list of strings, as in:'''),

    P(SPAN("query_vars = ['product', 'inventory', 'cost']",
           klass='codesample')),

    P("""This will create three instance variables for each request
      and set them to the values specified in the URL or form, OR, if
      the names are not in the URL or form, they will be set to an
      empty string.  This is equivalent to specifying:"""),

    P(SPAN("""query_vars = [('product', ''), ('inventory', ''),
              ('cost', '')]""", klass='codesample')),

    P("""The complete specification for an element in C{query_vars}
      is:"""),
    
    P("C{NAME | (NAME, DEFAULT [, CONVERSION])}",
      style='margin-left:.25in'),

    P('''C{NAME} must be a string and a legal python identifier (as it
      will become the name of an instance variable). In the first
      form, where you only specify C{NAME}, if it is not in the
      request or is only whitespace (values are stripped of leading
      and trailing whitespace, so a value of only whitespace will
      become an empty string) then the value will be an empty
      string. The first form is equivalent to:'''),

    P("C{(NAME, '', None)}", style='margin-left:.25in'),

    P('''In the second form you can specify a default value if the
      query variable does not appear in the request (or is only
      whitespace) and, optionally, a conversion function.  C{DEFAULT}
      must be a string and specifies the default value. C{CONVERSION}
      must be C{None} (the default, which does no conversion) or be a
      callable that accepts a single string argument and returns the
      converted value.  If the conversion function raises an exception
      the instance variable will be set to its default value.'''),

    P('''The following form demonstrates converting form variables by
      setting C{DEFAULT} and C{CONVERSION}:'''))
