# -*- python -*-

from _TutorialBase import *

class lists(TutorialBase):

    title = 'Converting URL/Form Data to Python Lists'
    css_links = ['tutorial.css']
    all_colors = ['red', 'blue', 'green', 'violet', 'cyan', 'amber']
    all_animals = ['dog', 'cat', 'African Sparrow', 'hippopotamus', 'grub',
                   'python']
    
    query_vars = [('colors', []),
                  ('animals', [])]

    def write_content(self):

        if self.req.method == 'POST':
            self.writeln('Show me the ', A('overview', href=self.req.uri),
                         ' again.')
            self.process_form()
        else:
            self.writeln(expand_codes(overview))

        self.writeln(P(self.theform()))
        

    def process_form(self):
        div = DIV(klass='formoutput')
        div.append(STRONG('This servlet has been posted via a form:'),
                   BR(), BR())
        for name, lst in [('colors', self.colors),
                          ('animals', self.animals)]:
            if lst:
                div.append('Your favorite %s are: %s'
                           % (name, ', '.join(lst)))
            else:
                div.append("You don't have any favorite %s!  :-(" % name)
            div.append(BR())

        self.writeln(CENTER(div))

    def theform(self):

        colorcell = TD()
        for color in self.all_colors:
            colorcell.append(INPUT(type='checkbox',
                                   name='colors',
                                   value=color,
                                   checked=color in self.colors),
                             ' %s' % color, BR())

        animalselect = SELECT(name='animals', multiple=True,
                              size=len(self.all_animals))
        for animal in self.all_animals:
            animalselect.append(OPTION(animal,
                                       selected=animal in self.animals))

        table = TABLE(TR(TD('Favorite Colors:'),
                         colorcell,
                         TD('&nbsp;'*20),
                         TD('Favorite Animals:'),
                         TD(animalselect),
                         valign='top'),
                      
                      TR(TD(INPUT(type='submit'),
                            colspan=5, align='center')),
                      klass='formdisplay')
        
        return FORM(CENTER(table), method='POST')


overview = DIV(

    '''I lied in the previous servlet.  I said C{DEFAULT} had to be a
    string.  Actually, C{DEFAULT} can be a string, list or dict and
    its type defines how the data is extracted from the C{form}
    instance variable.  If C{DEFAULT} is a string, the value will be
    retrieved with a call to C{form.getfirst()}.  If C{DEFAULT} is a
    list, the value will be retrieved with a call to
    C{form.getlist()}.  If C{DEFAULT} is a dict, C{form} will be
    searched for variables with names of the form E{NAME[KEY]}.''',

    P('''If you are not familiar with the C{getfirst} and C{getlist}
      methods, you should review the standard mod_python
      documentation; see mod_python.utils.FieldStorage.'''),

    P('''This servlet will demonstrate lists.  The next will
      demonstrate dicts.'''),

    P('''When C{DEFAULT} specifies a list, it can be empty or have
      elements, but if it has elements they must be strings.  You may
      also specify C{CONVERSION}, in which case it should be a
      callable that expects a list as its only argument; it may return
      any arbitrary object, but typically you will want it to return a
      list with its E{elements} converted to some arbitrary type,
      e.g., convert a list of strings into a list of ints.'''),

    P('''When do you want to use lists?  They are particularly handy
      when processing C{INPUT} elements with the S{type} attribute set
      to E{checkbox} and C{SELECT} elements with the S{multiple}
      attribute set.  Here is an example of each:'''))

